#
# fishing.R, 29 Jan 21


source("/usr1/rbook/examples/ESEUR_config.r")

library("lubridate")
# library("rugarch")


plot_layout(1, 2)

pal_col=rainbow(2)

# Look for significant changing variance in the series
variance_plot=function(df)
{
dcpan=diff(df)

acf((dcpan-mean(dcpan, na.rm=TRUE))^2, na.action=na.pass)
}


mkl_acf=function(lang, start_date, mod_str="")
{
t_mc=subset(mc, as.Date(start_date) <= date)

# Take log to handle the exponential growth in variance
acf(diff(log(unlist(t_mc[lang]))), na.action=na.pass, col=point_col)
text(20, 0.9, mod_str, cex=1.2)
}

mk_acf=function(lang, start_date, mod_str="")
{
t_mc=subset(mc, as.Date(start_date) <= date)

acf(diff(unlist(t_mc[lang])), na.action=na.pass, col=point_col)
text(20, 0.9, mod_str, cex=1.2)
}

mk_pacf=function(lang, start_date)
{
t_mc=subset(mc, as.Date(start_date) <= date)

pacf(diff(log(unlist(t_mc[lang]))), na.action=na.pass)
}


mc=read.csv("modulecounts.csv", as.is=TRUE)
mc$date=as.Date(mc$date, format="%Y/%m/%d")
mc$wday=wday(mc$date, week_start=1)


mean(diff(mc$CPAN), na.rm=TRUE)
mean(diff(mc$PyPI), na.rm=TRUE)
mean(diff(mc$Maven.Central..Java.), na.rm=TRUE)
mean(diff(mc$Rubygems.org), na.rm=TRUE)
mean(diff(mc$Perl.6.Ecosystem..perl.6.), na.rm=TRUE)
mean(diff(mc$CRAN..R.), na.rm=TRUE)

# acf(diff(mc$CPAN), na.action=na.pass)
mkl_acf("PyPI", "2010-01-01", "PyPI")
mk_acf("Maven.Central..Java.", "2014-01-01", "Maven Central (Java)")
# mk_acf("Crates.io..Rust.", "2015-01-01")
# mk_acf("nuget...NET.", "2012-01-01")
# mk_pacf("Packagist..PHP.", "2013-01-01")
# mk_pacf("Rubygems.org", "2011-01-01")



# py=mc$PyPI
# py[is.na(py)]=mean(py, na.rm=TRUE)
# lag.plot(py, lags=6)

# variance_plot(mc$CPAN)
variance_plot(mc$PyPI)
# variance_plot(mc$npm..node.js.)

start_date=as.Date("2010-01-01")

plot(mc$date, c(1, diff(mc$PyPI)), log="y", col=pal_col[2],
	xaxs="i",
	xlab="Date", ylab="Daily change\n")
text(start_date+1e3, 1000, "PyPI", cex=1.2)

mc$Ndays=as.numeric(mc$date-start_date)
mc_mod=glm(log(PyPI) ~ Ndays+I(Ndays^2), data=mc)
# summary(mc_mod)
pred=exp(predict(mc_mod, newdata=mc))
lines(mc$date, c(10, diff(pred)), col=pal_col[1])

# py_coef=coef(mc_mod)
# exp(c(py_coef[2], py_coef[3]))

plot(mc$date, c(1, diff(mc$Rubygems.org)), ylim=c(1, 150), col=point_col,
	xaxs="i",
	xlab="Date", ylab="Daily change\n")
text(start_date+1e3, 150, "Rubygems", cex=1.2)

# plot(mc$date, c(1, diff(mc$CPAN)),
# 	ylim=c(-30, 50))
# plot(mc$date, c(1, diff(mc$CRAN..R.)), log="y")
# plot(mc$date, c(1, diff(mc$Crates.io..Rust.)), log="y")
# plot(mc$date, c(1, diff(mc$Maven.Central..Java.)),
# 	ylim=c(-30, 350))
# plot(mc$date, c(1, diff(mc$nuget...NET.)), log="y")
# plot(mc$date, c(1, diff(mc$Packagist..PHP.)), log="y")

pacf(diff(mc$PyPI), na.action=na.pass)

start_date=as.Date("2010-01-01")
rad_per_day=(2*pi)/365
mc$Ndays=as.numeric(mc$date-start_date)
mc$rad_Ndays=rad_per_day*as.numeric(mc$date-start_date)
mc$Ndays=as.numeric(mc$date-start_date)
# mc_mod=glm(log(PyPI) ~ Ndays+I(Ndays^2)+sin(rad_Ndays)+cos(rad_Ndays), data=mc)
mc_mod=glm(log(PyPI) ~ Ndays+I(Ndays^2), data=mc)
summary(mc_mod)

mc_coef=coef(mc_mod)
# start of cycle
atan(mc_coef[4]/mc_coef[5])
# peaks at:
sqrt(mc_coef[4]^2+mc_coef[5]^2)

# x_vals=seq(0, 2*pi, by=0.01)
# plot(x_vals, mc_coef[4]*sin(x_vals)+mc_coef[5]*cos(x_vals))


# Break down into trend, season and random components
season_mod=function(df, freq)
{
hr_ts=ts(df, start=c(0, 0), frequency=freq)
plot(stl(hr_ts, s.window="periodic", na.action=na.contiguous))
}

season_mod(log(mc$PyPI), 7)


py_days=daply(mc, .(wday), function(df) sum(df$PyPI, na.rm=TRUE))
jv_days=daply(mc, .(wday), function(df) sum(df$Maven.Central..Java., na.rm=TRUE))

#
g_spec=ugarchspec(mean.model=list(include.mean=TRUE, armaOrder=c(0, 1)))

pypi=subset(mc, (as.Date("2011-01-01") < date) & (date < as.Date("2020-01-01")))

pypid=c(20, diff(mc$PyPI))

py_mod=glm(log(pypid) ~ date, data=mc)
# summary(py_mod)
pred=exp(predict(py_mod, newdata=mc))

mc_na=is.na(pypid)
pypid[mc_na]=pred[mc_na]

ug_mod=ugarchfit(spec=g_spec, data=pypid)
ug_mod


par(cex=0.9)
par(cex.axis=1.2)
par(cex.lab=1.2)

dev.copy(device=png, file="/usr1/rbook/t.png")
dev.off()

